unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Menus, ShellAPI, ExtCtrls;

type
  TfmMain = class(TForm)
    Edit1: TEdit;
    ListBox1: TListBox;
    PopupMenu: TPopupMenu;
    Remove1: TMenuItem;
    filename1: TMenuItem;
    Open1: TMenuItem;
    N1: TMenuItem;
    Rename1: TMenuItem;
    AddNew1: TMenuItem;
    N2: TMenuItem;
    OpenDialog1: TOpenDialog;
    SearchButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ListBox1DrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure Edit1Change(Sender: TObject);
    procedure ListBox1DblClick(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure ListBox1ContextPopup(Sender: TObject; MousePos: TPoint;
      var Handled: Boolean);
    procedure AddNew1Click(Sender: TObject);
    procedure Remove1Click(Sender: TObject);
    procedure filename1Click(Sender: TObject);
    procedure Rename1Click(Sender: TObject);
    procedure ListBox1KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Edit1KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    //function GetDatasheetIdx: integer;
    procedure FormShow(Sender: TObject);
    function GetRelativePath(Path: String): String;
    procedure SearchButtonClick(Sender: TObject);
    procedure AcceptFiles(var msg: TMessage); message WM_DROPFILES;
  private
    { Private declarations }
  public
    { Public declarations }
  end;
  TStr = class
  public
    Name, Value: String;
    Index: Integer;
    constructor Create(AName, AValue: String; AIndex: Integer);
  end;

const
  DataFile = 'datasheets.dat';
  SearchURL = 'http://www.alldatasheet.com/view.jsp?Searchword=';

var
  fmMain: TfmMain;
  Datasheets: TStringList;

implementation

{$R *.dfm}

constructor TStr.Create(AName, AValue: String; AIndex: Integer);
begin
  Name := AName;
  Value := AValue;
  Index := AIndex;
end;

procedure TfmMain.AcceptFiles(var msg: TMessage);
const
  cnMaxFileNameLen = 255;
var
  i,
  nCount     : integer;
  acFileName : array [0..cnMaxFileNameLen] of char;

  fname, s: String;
begin
  // find out how many files we're accepting
  nCount := DragQueryFile(msg.WParam, $FFFFFFFF, acFileName, cnMaxFileNameLen);

  // query Windows one at a time for the file name
  for i := 0 to nCount-1 do
  begin
    DragQueryFile(msg.WParam, i, acFileName, cnMaxFileNameLen);

    // do your thing with the acFileName
    fname := GetRelativePath(acFileName);
    s := ChangeFileExt(ExtractFileName(fname),''); //Get name of file without extention
    Datasheets.Add(s+'='+fname);

  end;

  Datasheets.SaveToFile(DataFile);
  Edit1Change(nil);

  DragFinish(msg.WParam);
end;

function TfmMain.GetRelativePath(Path: String): String;
var
  AppPath: String;
begin
  AppPath := ExtractFilePath(ParamStr(0));
  if Pos(AppPath,Path) = 1 then
    result := Copy(Path,Length(AppPath)+1,Length(Path))
  else
    result := Path;
end;

{function TfmMain.GetDatasheetIdx: integer;
var
  i: integer;
begin
  //Rename the name of the datasheet
  result := -1;
  if ListBox1.ItemIndex < 0 then
    exit;
  for i := 0 to Datasheets.Count - 1 do
    if Datasheets.Names[i] = ListBox1.Items.Strings[ListBox1.ItemIndex] then
    begin
      result := i;
      break;
    end;
end;  }

procedure TfmMain.FormCreate(Sender: TObject);
begin
  //Load the datasheets
  Datasheets := TStringList.Create;
  Datasheets.Clear;
  if not FileExists(DataFile) then
    Datasheets.SaveToFile(DataFile);  //Save a blank file
  Datasheets.LoadFromFile(DataFile);
  //Mask box is blank, show all datasheets in the list
  Edit1Change(nil);

  if not GlassFrame.Enabled then
  begin
    Edit1.Left := Edit1.Left + 4;
    Edit1.Width := Edit1.Width - 4;
  end;

  DragAcceptFiles(Handle, True);
end;

procedure TfmMain.SearchButtonClick(Sender: TObject);
begin
  //Search for the datasheet online
  ShellExecute(0,'open',PChar(SearchURL + Edit1.Text),'','',SW_SHOW);
  Close;
end;

procedure TfmMain.FormDestroy(Sender: TObject);
begin
  Datasheets.Free;
end;

procedure TfmMain.FormShow(Sender: TObject);
begin
  Edit1.SetFocus;
end;

procedure TfmMain.Edit1Change(Sender: TObject);
  procedure AddItem(Name, Value: String; Index: Integer; Cmp: String);
  var
    s,s2: String;
  begin
    //We can have multiple entries for the same datasheet by separating them with a /
    s := Name + '/';
    while (s <> '') do
    begin
      s2 := Copy(s,1,Pos('/',s)-1);                                    //Separate
      if (Cmp = '') or (Pos(LowerCase(Cmp),LowerCase(s2)) > 0) then    //Filter results to a mask if it's not blank
        ListBox1.Items.AddObject(s2,TStr.Create(Name,Value,Index));
      Delete(s,1,Pos('/',s));
    end;
  end;
var
  i: integer;
begin
  //Only show items in the list box that match the mask in the edit box
  ListBox1.Items.BeginUpdate;  //BeginUpdate makes the items not be redrawn when added
  ListBox1.Items.Clear;
  //For each item that matches the mask, add it to the list
  for i := 0 to Datasheets.Count - 1 do
    AddItem(Datasheets.Names[i],Datasheets.Values[Datasheets.Names[i]],i,Edit1.Text);
  ListBox1.Items.EndUpdate;
end;

procedure TfmMain.Edit1KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = vk_return then
  begin
    if ListBox1.Items.Count = 0 then
      //exit;
      SearchButtonClick(nil);
    if ListBox1.ItemIndex < 0 then
      ListBox1.ItemIndex := 0;
    Open1Click(nil);
  end;
end;

procedure TfmMain.ListBox1ContextPopup(Sender: TObject; MousePos: TPoint;
  var Handled: Boolean);
begin
  ListBox1.ItemIndex := ListBox1.ItemAtPos(MousePos,true);
  Open1.Visible := ListBox1.ItemIndex >= 0;
  Rename1.Visible := Open1.Visible;
  Remove1.Visible := Open1.Visible;
  filename1.Visible := Open1.Visible;
  AddNew1.Default := not Open1.Visible;
  Open1.Default := Open1.Visible;
end;

procedure TfmMain.ListBox1DblClick(Sender: TObject);
begin
  //Open the Datasheet (default popup menu action)
  if ListBox1.ItemIndex >= 0 then
    Open1Click(nil)
  else
    AddNew1Click(nil);
end;

procedure TfmMain.ListBox1DrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  i,x,p1,p2: integer;
  s: string;
begin
  //Draw each list item and draw the letters bold
  //if it matches the search term in the edit box
with TListBox(Control) do
begin
  Canvas.FillRect(Rect);
  Canvas.Brush.Style := bsClear;

  x := Rect.Left+2;
  s := Items.Strings[Index];

  //Find position of the matching string
  p1 := Pos(LowerCase(Edit1.Text),LowerCase(s));
  if Length(Edit1.Text) = 0 then
    p1 := 0;
  p2 := p1;
  if p1 > 0 then
    p2 := p1 + Length(Edit1.Text);

  //Go through each character
  for i := 1 to Length(s)  do
  begin
    Canvas.Font.Style := [];
    //If character is in range, draw it bold
    if (i >= p1) and (i < p2) then
      Canvas.Font.Style := [fsBold];
    //Draw character and increment x for next character
    Canvas.TextOut(x,Rect.Top,s[i]);
    x := x + Canvas.TextWidth(s[i]);
  end;
end;
end;

procedure TfmMain.ListBox1KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = vk_delete then
  begin
    if ListBox1.ItemIndex >= 0 then
      Remove1Click(nil);
  end;
end;

procedure TfmMain.Open1Click(Sender: TObject);
var
  obj: TObject;
begin
  if ListBox1.ItemIndex = -1 then exit;

  //Open the Datasheet (default popup menu action)
  obj := ListBox1.Items.Objects[ListBox1.ItemIndex];
  if Assigned(obj) and (TStr(Obj).Value <> '') then
  begin
    //Open Datasheet (Shell Context Menu) and Close the program
    ShellExecute(0,'Open',PChar(TStr(Obj).Value),'','',SW_SHOWNORMAL);
    Close;
    exit;
  end else
    MessageDlg('Error: No associated datasheet',mtError,[mbOK],0);
end;

procedure TfmMain.Remove1Click(Sender: TObject);
var
  obj: TObject;
  mr: TModalResult;
  s,s2: String;
begin
  if ListBox1.ItemIndex = -1 then exit;

  //Delete the Datasheet
  obj := ListBox1.Items.Objects[ListBox1.ItemIndex];
  if Assigned(obj) then
  begin
    mr := mrYes; //Default to delete all aliases
    if Pos('/',TStr(Obj).Name) > 0 then
    begin
      //Multiple aliases to the same datasheet, ask which action to do
      mr := MessageDlg('This datasheet has multiple aliases. Would you like to delete all aliases?'#13#10'Yes: Delete all references'#13#10'No: Only delete this alias',mtConfirmation,[mbYes,mbNo,mbCancel],0);
    end;

    if mr = mrYes then
    begin
      //Delete all references to the datasheet
      Datasheets.Delete(TStr(Obj).Index);
      Datasheets.SaveToFile(DataFile);
      Edit1Change(nil);
    end
    else if mr = mrNo then
    begin
      //Only delete the selected alias (really a rename)
      s := TStr(Obj).Name;
      s2 := ListBox1.Items.Strings[ListBox1.ItemIndex];

      Delete(s,Pos(s2,s),Length(s2)+1);

      Datasheets.Strings[TStr(Obj).Index] := s+'='+TStr(Obj).Value;
      Datasheets.SaveToFile(DataFile);
      Edit1Change(nil);
    end;
  end else
    MessageDlg('Error: No associated datasheet',mtError,[mbOK],0);
end;

procedure TfmMain.Rename1Click(Sender: TObject);
var
  obj: TObject;
  s: string;
begin
  if ListBox1.ItemIndex = -1 then exit;

  //Rename the datasheet
  obj := ListBox1.Items.Objects[ListBox1.ItemIndex];
  if Assigned(obj) then
  begin
    s := TStr(Obj).Name;
    s := InputBox('Rename Datasheet','Enter the name of the Datasheet'#13#10'Separate multiple entries with a /',s);
    if (s <> '') and (s <> TStr(Obj).Name) then
    begin
      Datasheets.Strings[TStr(Obj).Index] := s+'='+TStr(Obj).Value;
      Datasheets.SaveToFile(DataFile);
      Edit1Change(nil);
    end;
  end else
    MessageDlg('Error: No associated datasheet',mtError,[mbOK],0);
end;

procedure TfmMain.filename1Click(Sender: TObject);
var
  obj: TObject;
  fname: String;
begin
  if ListBox1.ItemIndex = -1 then exit;

  //Change the filename of a datasheet
  obj := ListBox1.Items.Objects[ListBox1.ItemIndex];
  if Assigned(obj) then
  begin
    OpenDialog1.Title := 'Change Datasheet File';
    OpenDialog1.FileName := TStr(Obj).Value;
    if OpenDialog1.Execute then
    begin
      fname := GetRelativePath(OpenDialog1.FileName);
      Datasheets.Values[Datasheets.Names[TStr(Obj).Index]] := fname;
      Datasheets.SaveToFile(DataFile);
      Edit1Change(nil);
    end;
  end else
    MessageDlg('Error: No associated datasheet',mtError,[mbOK],0);
end;

procedure TfmMain.AddNew1Click(Sender: TObject);
var
  s,s2,fname: string;
begin
  OpenDialog1.FileName := '';
  OpenDialog1.Title := 'Add New Datasheet';
  if OpenDialog1.Execute then
  begin
    fname := GetRelativePath(OpenDialog1.FileName);
    s := ChangeFileExt(ExtractFileName(fname),''); //Get name of file without extention
    s2 := s;
    s := InputBox('Add New Datasheet','Enter the name of the new Datasheet'#13#10'This is used for displaying & searching',s);
    if (s <> '') and (s <> s2) then
    begin
      Datasheets.Add(s+'='+fname);
      Datasheets.SaveToFile(DataFile);
      Edit1Change(nil);
    end;
  end;
end;

end.
